@@ -48,6 +48,8 @@ extern gboolean src_use_braille;
 extern gboolean src_use_braille_monitor;
 extern gboolean src_enable_format_string;
 
+gchar    *scroll = NULL;
+
 gboolean  src_button_left_pressed   = FALSE;
 gboolean  src_button_middle_pressed = FALSE;
 gboolean  src_button_right_pressed  = FALSE;
@@ -78,32 +80,31 @@ src_mag_lock_xy_scale ()
 	    lock  = !lock;
 	    if (lock)
 	    {
-		gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
-		gint zoom_factory;
-		gint zoom_factorx;
+		gdouble default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
+		gdouble zoom_factory;
+		gdouble zoom_factorx;
 		if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_Y, 
-						   CFGT_INT, 
+						   CFGT_FLOAT, 
 						   &zoom_factory, 
 						   (gpointer)&default_val,
 						   MAGNIFIER_CONFIG_PATH)
 		    ) return FALSE;
 		if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_X, 
-						   CFGT_INT, 
+						   CFGT_FLOAT, 
 						   &zoom_factorx, 
 						   (gpointer)&default_val,
 						   MAGNIFIER_CONFIG_PATH)
 		    ) return FALSE;
 		
-		zoom_factorx = 
-		    zoom_factory = (zoom_factory + zoom_factorx) >> 1;
+		zoom_factorx =  zoom_factory = (zoom_factory + zoom_factorx) / 2;
 
 		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-				      CFGT_INT, 
+				      CFGT_FLOAT, 
 				      &zoom_factory,
 				      MAGNIFIER_CONFIG_PATH)
 		    ) return FALSE;
 		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-				      CFGT_INT, 
+				      CFGT_FLOAT, 
 				      &zoom_factory,
 				      MAGNIFIER_CONFIG_PATH)
 		    ) return FALSE;
@@ -129,13 +130,13 @@ src_mag_increase_y_scale ()
 	}
 	else
 	{
-	    gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
-	    gint zoom_factory;
+	    gdouble default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
+	    gdouble zoom_factory;
 	    gboolean zoom_factor_lock;
 	    gboolean zoom_factor_lock_def = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;
 
 	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_Y, 
-					       CFGT_INT, 
+					       CFGT_FLOAT, 
 					       &zoom_factory, 
 					       (gpointer)&default_val,
 					       MAGNIFIER_CONFIG_PATH)
@@ -152,7 +153,7 @@ src_mag_increase_y_scale ()
 	    if (!zoom_factor_lock)
 	    {
 		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-				      CFGT_INT, 
+				      CFGT_FLOAT, 
 				      &zoom_factory,
 				      MAGNIFIER_CONFIG_PATH)
 		   ) return FALSE;
@@ -160,12 +161,12 @@ src_mag_increase_y_scale ()
 	    else
 	    {
 		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-				      CFGT_INT, 
+				      CFGT_FLOAT, 
 				      &zoom_factory,
 				      MAGNIFIER_CONFIG_PATH)
 		   ) return FALSE;
 		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-				      CFGT_INT, 
+				      CFGT_FLOAT, 
 				      &zoom_factory,
 				      MAGNIFIER_CONFIG_PATH)
 		   ) return FALSE;
@@ -187,13 +188,13 @@ src_mag_decrease_y_scale ()
 	}
 	else
 	{
-	    gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
-	    gint zoom_factory;
+	    gdouble default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
+	    gdouble zoom_factory;
 	    gboolean zoom_factor_lock;
 	    gboolean zoom_factor_lock_def = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;
 
 	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_Y, 
-					       CFGT_INT, 
+					       CFGT_FLOAT, 
 					       &zoom_factory, 
 					       (gpointer)&default_val,
 					       MAGNIFIER_CONFIG_PATH)
@@ -210,7 +211,7 @@ src_mag_decrease_y_scale ()
 	    if (!zoom_factor_lock)
 	    {
 		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-				      CFGT_INT, 
+				      CFGT_FLOAT, 
 				      &zoom_factory,
 				      MAGNIFIER_CONFIG_PATH)
 		   ) return FALSE;
@@ -218,12 +219,12 @@ src_mag_decrease_y_scale ()
 	    else
 	    {
 		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-				      CFGT_INT, 
+				      CFGT_FLOAT, 
 				      &zoom_factory,
 				      MAGNIFIER_CONFIG_PATH)
 		   ) return FALSE;
 		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-				      CFGT_INT, 
+				      CFGT_FLOAT, 
 				      &zoom_factory,
 				      MAGNIFIER_CONFIG_PATH)
 		   ) return FALSE;
@@ -246,13 +247,13 @@ src_mag_increase_x_scale ()
 	}
 	else
 	{
-	    gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
-	    gint zoom_factorx;
+	    gdouble default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
+	    gdouble zoom_factorx;
 	    gboolean zoom_factor_lock;
 	    gboolean zoom_factor_lock_def = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;
 
 	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_X, 
-					       CFGT_INT, 
+					       CFGT_FLOAT, 
 					       &zoom_factorx, 
 					       (gpointer)&default_val,
 					       MAGNIFIER_CONFIG_PATH)
@@ -269,7 +270,7 @@ src_mag_increase_x_scale ()
 	    if (!zoom_factor_lock)
 	    {
 	        if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-				      CFGT_INT, 
+				      CFGT_FLOAT, 
 				      &zoom_factorx,
 				      MAGNIFIER_CONFIG_PATH)
 		   ) return FALSE;
@@ -277,12 +278,12 @@ src_mag_increase_x_scale ()
     	    else
 	    {
 		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-				      CFGT_INT, 
+				      CFGT_FLOAT, 
 				      &zoom_factorx,
 				      MAGNIFIER_CONFIG_PATH)
 		   ) return FALSE;
 		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-				      CFGT_INT, 
+				      CFGT_FLOAT, 
 				      &zoom_factorx,
 				      MAGNIFIER_CONFIG_PATH)
 	    	   ) return FALSE;
@@ -304,13 +305,13 @@ src_mag_decrease_x_scale ()
 	}
         else
 	{	
-            gint default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
-	    gint zoom_factorx;
+            gdouble default_val = DEFAULT_MAGNIFIER_ZOOM_FACTOR_XY;
+	    gdouble zoom_factorx;
 	    gboolean zoom_factor_lock;
 	    gboolean zoom_factor_lock_def = DEFAULT_MAGNIFIER_ZOOM_FACTOR_LOCK;
 
 	    if (!srconf_get_data_with_default (MAGNIFIER_ZOOM_FACTOR_X, 
-					       CFGT_INT, 
+					       CFGT_FLOAT, 
 					       &zoom_factorx, 
 					       (gpointer)&default_val,
 					       MAGNIFIER_CONFIG_PATH)
@@ -327,7 +328,7 @@ src_mag_decrease_x_scale ()
 	    if (!zoom_factor_lock)
 	    {
 		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-				      CFGT_INT, 
+				      CFGT_FLOAT, 
 				      &zoom_factorx,
 				      MAGNIFIER_CONFIG_PATH)
 		   ) return FALSE;
@@ -335,12 +336,12 @@ src_mag_decrease_x_scale ()
 	    else
 	    {
 		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_Y,
-				      CFGT_INT, 
+				      CFGT_FLOAT, 
 				      &zoom_factorx,
 				      MAGNIFIER_CONFIG_PATH)
 		   ) return FALSE;
 		if (!srconf_set_data (MAGNIFIER_ZOOM_FACTOR_X,
-				      CFGT_INT, 
+				      CFGT_FLOAT, 
 				      &zoom_factorx,
 				      MAGNIFIER_CONFIG_PATH)
 		   ) return FALSE;
@@ -2461,12 +2462,27 @@ src_present_line_for_speech (SRWAccLine *line)
 
         cell = g_array_index (line->srw_acc_line, SRWAccCell *, i);
 	    
-        if (cell->id > 0)
+        switch (cell->id)
 	{
-	    gchar *tmp;
-	    tmp = spout;
-	    spout = g_strconcat (spout, cell->ch, NULL);
-	    g_free (tmp);
+	    case SRW_DELIMITER_CELL:
+	    {
+		gchar *tmp;
+		tmp = spout;
+		spout = g_strconcat (spout, " ", NULL);
+		g_free (tmp);
+	    };
+	    break;
+	    default: 
+	    {
+		if (cell->id > 0)
+		{
+		    gchar *tmp;
+		    tmp = spout;
+		    spout = g_strconcat (spout, cell->ch, NULL);
+		    g_free (tmp);
+		}    
+	    };
+	    break;
 	}
     }
     
@@ -2564,9 +2580,23 @@ src_present_line_to_braille_or_brlmon (SRWAccLine *line)
 	sprintf (offset, "%d", braille_offset);
 	brlout = g_strconcat ("<BRLOUT language=\"", braille_translation_table, "\" bstyle=\"8\" clear=\"true\">",
 				"<BRLDISP role=\"main\" offset=\"", offset, "\" clear=\"true\">",
-				brlout, "</BRLDISP>",
-				"</BRLOUT>",
+				brlout, 
 				NULL);
+	if (scroll)
+	{
+	    brlout = g_strconcat (brlout, 
+	    	            	  "<SCROLL mode=\"", scroll, "\"></SCROLL>",
+				  "</BRLDISP>",
+				  "</BRLOUT>",
+				  NULL);
+	}  
+	else
+	{
+	    brlout = g_strconcat (brlout, 
+				  "</BRLDISP>",
+				  "</BRLOUT>",
+				  NULL);
+	}			
 	g_free (tmp);
 	if (src_use_braille)
 	    src_braille_send (brlout);
@@ -3086,9 +3116,9 @@ static gboolean
 src_braille_scroll_char_left ()
 {
     gboolean old = src_use_speech;
-    gint offset;
-    offset = src_braille_get_offset ();
-    src_braille_set_offset (MAX (0, offset - 1));
+    if (scroll)
+	g_free (scroll);
+    scroll = g_strdup ("-1");
     src_use_speech = FALSE;
     src_cmd_queue_add ("repeat last");
     src_cmd_queue_process ();
@@ -3100,10 +3130,9 @@ static gboolean
 src_braille_scroll_char_right ()
 {
     gboolean old = src_use_speech;
-    gint offset;
-    
-    offset = src_braille_get_offset ();
-    src_braille_set_offset (MIN (1024, offset + 1));
+      if (scroll)
+	g_free (scroll);
+    scroll = g_strdup ("1");
     src_use_speech = FALSE;
     src_cmd_queue_add ("repeat last");
     src_cmd_queue_process ();
@@ -3117,10 +3146,9 @@ static gboolean
 src_braille_scroll_display_left ()
 {
     gboolean old = src_use_speech;
-    gint offset;
-    
-    offset = src_braille_get_offset ();
-    src_braille_set_offset (MAX (0, offset - 20));
+    if (scroll)
+	g_free (scroll);
+    scroll = g_strdup ("-width");
     src_use_speech = FALSE;
     src_cmd_queue_add ("repeat last");
     src_cmd_queue_process ();
@@ -3134,10 +3162,9 @@ static gboolean
 src_braille_scroll_display_right ()
 {
     gboolean old = src_use_speech;
-    gint offset;
-    
-    offset = src_braille_get_offset ();
-    src_braille_set_offset (MIN (1024, offset + 20));
+    if (scroll)
+	g_free (scroll);
+    scroll = g_strdup ("width");
     src_use_speech = FALSE;
     src_cmd_queue_add ("repeat last");
     src_cmd_queue_process ();
